@@ -4,14 +4,14 @@
 # http://www.opensource.org/licenses/mit-license.html
 # See license.txt for more details.
 
-from shutil import rmtree
-from tempfile import TemporaryFile,mkdtemp
+from mock import Mock
+from StringIO import StringIO
+from tempfile import TemporaryFile
+from testfixtures import compare,Comparison as C,should_raise,replace,log_capture,tempdir
 from unittest import TestSuite,TestCase,makeSuite
 from xlrd import open_workbook,XL_CELL_NUMBER,XL_CELL_ERROR,XL_CELL_TEXT
 from xlutils.filter import BaseReader,GlobReader,MethodFilter,BaseWriter,process
 from xlutils.tests.fixtures import test_files,test_xls_path,make_book,make_sheet,DummyBook
-from xlutils.tests.fixtures import compare,O,TestCallable,TestCallableMethod,TestTraversable
-from zope.testing.loggingsupport import InstalledHandler
 
 import os
 
@@ -40,9 +40,9 @@ class TestBaseReader(TestCase):
 
     def test_no_implementation(self):
         r = BaseReader()
-        f = TestCallable()
-        self.assertRaises(NotImplementedError,r,f)
-        self.assertEqual(f.called,[])
+        f = Mock()
+        should_raise(r,NotImplementedError())(f)
+        self.assertEqual(f.method_calls,[('start',(),{})])
         
     def test_custom_filepaths(self):
         # also tests the __call__ method
@@ -50,30 +50,30 @@ class TestBaseReader(TestCase):
             def get_filepaths(self):
                 return (test_xls_path,)
         t = TestReader()
-        f = TestCallable()
+        f = Mock()
         t(f)
-        f.compare(self,[
-            ('workbook',(O('xlrd.Book'),'test.xls')),
-            ('sheet',(O('xlrd.sheet.Sheet'),u'Sheet1')),
-            ('row',(0,0)),
-            ('cell',(0,0,0,0)),
-            ('cell',(0,1,0,1)),
-            ('row',(1,1)),
-            ('cell',(1,0,1,0)),
-            ('cell',(1,1,1,1)),
-            ('sheet',(O('xlrd.sheet.Sheet'),u'Sheet2')),
-            ('row',(0,0)),
-            ('cell',(0,0,0,0)),
-            ('cell',(0,1,0,1)),
-            ('row',(1,1)),
-            ('cell',(1,0,1,0)),
-            ('cell',(1,1,1,1)),
-            ('finish',()),
+        compare(f.method_calls,[
+            ('start',(),{}),
+            ('workbook',(C('xlrd.Book',
+                           pickleable=0,
+                           formatting_info=1,
+                           strict=False),'test.xls'),{}),
+            ('sheet',(C('xlrd.sheet.Sheet'),u'Sheet1'),{}),
+            ('row',(0,0),{}),
+            ('cell',(0,0,0,0),{}),
+            ('cell',(0,1,0,1),{}),
+            ('row',(1,1),{}),
+            ('cell',(1,0,1,0),{}),
+            ('cell',(1,1,1,1),{}),
+            ('sheet',(C('xlrd.sheet.Sheet'),u'Sheet2'),{}),
+            ('row',(0,0),{}),
+            ('cell',(0,0,0,0),{}),
+            ('cell',(0,1,0,1),{}),
+            ('row',(1,1),{}),
+            ('cell',(1,0,1,0),{}),
+            ('cell',(1,1,1,1),{}),
+            ('finish',(),{}),
             ])
-        # check we're opening things correctly
-        book = f.called[0][1][0]
-        self.assertEqual(book.pickleable,0)
-        self.assertEqual(book.formatting_info,1)
 
     def test_custom_getworkbooks(self):
         book = make_book((('1','2','3'),))
@@ -81,62 +81,69 @@ class TestBaseReader(TestCase):
             def get_workbooks(self):
                 yield book,'test.xls'
         t = TestReader()
-        f = TestCallable()
+        f = Mock()
         t(f)
-        f.compare(self,[
-            ('workbook',(O('xlutils.tests.fixtures.DummyBook'),'test.xls')),
-            ('sheet',(O('xlrd.sheet.Sheet'),'test sheet')),
-            ('row',(0,0)),
-            ('cell',(0,0,0,0)),
-            ('cell',(0,1,0,1)),
-            ('cell',(0,2,0,2)),
-            ('finish',()),
+        compare(f.method_calls,[
+            ('start',(),{}),
+            ('workbook',(C('xlutils.tests.fixtures.DummyBook'),'test.xls'),{}),
+            ('sheet',(C('xlrd.sheet.Sheet'),'test sheet'),{}),
+            ('row',(0,0),{}),
+            ('cell',(0,0,0,0),{}),
+            ('cell',(0,1,0,1),{}),
+            ('cell',(0,2,0,2),{}),
+            ('finish',(),{}),
             ])
         # check we're getting the right things
-        self.failUnless(f.called[0][1][0] is book)
-        self.failUnless(f.called[1][1][0] is book.sheet_by_index(0))
+        self.failUnless(f.method_calls[1][1][0] is book)
+        self.failUnless(f.method_calls[2][1][0] is book.sheet_by_index(0))
     
 class TestBaseFilter(TestCase):
 
     def setUp(self):
         from xlutils.filter import BaseFilter
         self.filter = BaseFilter()
-        self.filter.next = self.tf = TestCallable()
+        self.filter.next = self.tf = Mock()
 
+    def test_start(self):
+        self.filter.start()
+        self.assertEqual(self.tf.method_calls,[
+            ('start',(),{})
+            ])
+                         
     def test_workbook(self):
         self.filter.workbook('rdbook','wtbook_name')
-        self.assertEqual(self.tf.called,[
-            ('workbook',('rdbook','wtbook_name'))
+        self.assertEqual(self.tf.method_calls,[
+            ('workbook',('rdbook','wtbook_name'),{})
             ])
                          
     def test_sheet(self):
         self.filter.sheet('rdsheet','wtsheet_name')
-        self.assertEqual(self.tf.called,[
-            ('sheet',('rdsheet','wtsheet_name'))
+        self.assertEqual(self.tf.method_calls,[
+            ('sheet',('rdsheet','wtsheet_name'),{})
             ])
                          
     def test_set_rdsheet(self):
         self.filter.set_rdsheet('rdsheet2')
-        self.assertEqual(self.tf.called,[
-            ('set_rdsheet',('rdsheet2',))
+        self.assertEqual(self.tf.method_calls,[
+            ('set_rdsheet',('rdsheet2',),{})
             ])
                          
     def test_row(self):
         self.filter.row(0,1)
-        self.assertEqual(self.tf.called,[
-            ('row',(0,1))
+        self.assertEqual(self.tf.method_calls,[
+            ('row',(0,1),{})
             ])
                          
     def test_cell(self):
         self.filter.cell(0,1,2,3)
-        self.assertEqual(self.tf.called,[
-            ('cell',(0,1,2,3))
+        self.assertEqual(self.tf.method_calls,[
+            ('cell',(0,1,2,3),{})
             ])
                          
     def test_finish(self):
         self.filter.finish()
-        self.assertEqual(self.tf.called,[
-            ('finish',())
+        self.assertEqual(self.tf.method_calls,[
+            ('finish',(),{})
             ])
 
 class OurMethodFilter(MethodFilter):
@@ -152,14 +159,28 @@ class TestMethodFilter(TestCase):
         self.called = []
 
     def do_calls_and_test(self,filter):
-        filter.next = tf = TestCallable()
+        filter.next = tf = Mock()
+        filter.start()
         filter.workbook('rdbook','wtbook_name')
         filter.sheet('rdsheet','wtsheet_name')
         filter.row(0,1)
         filter.cell(0,1,2,3)
         filter.set_rdsheet('rdsheet2')
         filter.finish()
-        self.assertEqual(tf.called,[
+        self.assertEqual(tf.method_calls,[
+            ('start',(),{}),
+            ('workbook',('rdbook','wtbook_name'),{}),
+            ('sheet',('rdsheet','wtsheet_name'),{}),
+            ('row',(0,1),{}),
+            ('cell',(0,1,2,3),{}),
+            ('set_rdsheet',('rdsheet2',),{}),
+            ('finish',(),{}),
+            ])
+        
+    def test_all(self):
+        self.do_calls_and_test(OurMethodFilter(self.called))
+        compare(self.called,[
+            ('start',()),
             ('workbook',('rdbook','wtbook_name')),
             ('sheet',('rdsheet','wtsheet_name')),
             ('row',(0,1)),
@@ -167,10 +188,23 @@ class TestMethodFilter(TestCase):
             ('set_rdsheet',('rdsheet2',)),
             ('finish',()),
             ])
-        
-    def test_all(self):
-        self.do_calls_and_test(OurMethodFilter(self.called))
-        self.assertEqual(self.called,[
+
+    def test_all_text(self):
+        self.do_calls_and_test(OurMethodFilter(self.called,call_on='True'))
+        compare(self.called,[
+            ('start',()),
+            ('workbook',('rdbook','wtbook_name')),
+            ('sheet',('rdsheet','wtsheet_name')),
+            ('row',(0,1)),
+            ('cell',(0,1,2,3)),
+            ('set_rdsheet',('rdsheet2',)),
+            ('finish',()),
+            ])
+
+    def test_all_text_list(self):
+        self.do_calls_and_test(OurMethodFilter(self.called,call_on=['True']))
+        compare(self.called,[
+            ('start',()),
             ('workbook',('rdbook','wtbook_name')),
             ('sheet',('rdsheet','wtsheet_name')),
             ('row',(0,1)),
@@ -181,79 +215,76 @@ class TestMethodFilter(TestCase):
 
     def test_somecalls_and_test(self):
         self.do_calls_and_test(OurMethodFilter(self.called,['row','cell']))
-        self.assertEqual(self.called,[
+        compare(self.called,[
             ('row',(0,1)),
             ('cell',(0,1,2,3)),
             ])
 
     def test_none(self):
         self.do_calls_and_test(OurMethodFilter(self.called,()))
-        self.assertEqual(self.called,[])
+        compare(self.called,[])
+
+    def test_start(self):
+        self.do_calls_and_test(OurMethodFilter(self.called,['start']))
+        compare(self.called,[
+            ('start',()),
+            ])
 
     def test_workbook(self):
         self.do_calls_and_test(OurMethodFilter(self.called,['workbook']))
-        self.assertEqual(self.called,[
+        compare(self.called,[
             ('workbook',('rdbook','wtbook_name')),
             ])
 
     def test_sheet(self):
         self.do_calls_and_test(OurMethodFilter(self.called,['sheet']))
-        self.assertEqual(self.called,[
+        compare(self.called,[
             ('sheet',('rdsheet','wtsheet_name')),
             ])
 
     def test_set_rdsheet(self):
         self.do_calls_and_test(OurMethodFilter(self.called,['set_rdsheet']))
-        self.assertEqual(self.called,[
+        compare(self.called,[
             ('set_rdsheet',('rdsheet2',)),
             ])
 
     def test_row(self):
         self.do_calls_and_test(OurMethodFilter(self.called,['row']))
-        self.assertEqual(self.called,[
+        compare(self.called,[
             ('row',(0,1)),
             ])
 
     def test_cell(self):
         self.do_calls_and_test(OurMethodFilter(self.called,['cell']))
-        self.assertEqual(self.called,[
+        compare(self.called,[
             ('cell',(0,1,2,3)),
             ])
 
     def test_finish(self):
         self.do_calls_and_test(OurMethodFilter(self.called,['finish']))
-        self.assertEqual(self.called,[
+        compare(self.called,[
             ('finish',()),
             ])
 
     def test_invalid(self):
-        self.assertRaises(Exception,OurMethodFilter,self.called,['foo'])
+        m = should_raise(OurMethodFilter,ValueError("'foo' is not a valid method name"))        
+        m(self.called,['foo'])
     
 from xlutils.filter import Echo
 
 class TestEcho(TestCase):
 
-    def test_method(self):
+    @replace('sys.stdout',StringIO())
+    def test_method(self,out):
         filter = Echo(methods=['workbook'])
-        from StringIO import StringIO
-        import sys
-        try:
-            sys.stdout = out = StringIO()
-            filter.method('name','foo',1)
-        finally:
-            sys.stdout = sys.__stdout__
-        self.assertEqual(out.getvalue(),"name:('foo', 1)\n")
+        filter.method('name','foo',1)
+        compare(out.getvalue(),"name:('foo', 1)\n")
         
-    def test_method_with_name(self):
+    @replace('sys.stdout',StringIO())
+    def test_method_with_name(self,out):
         filter = Echo('echo',['workbook'])
-        from StringIO import StringIO
-        import sys
-        try:
-            sys.stdout = out = StringIO()
-            filter.method('name','foo',1)
-        finally:
-            sys.stdout = sys.__stdout__
-        self.assertEqual(out.getvalue(),"'echo' name:('foo', 1)\n")
+        filter.method('name','foo',1)
+        compare(out.getvalue(),"'echo' name:('foo', 1)\n")
         
     def test_inheritance(self):
         self.failUnless(isinstance(Echo(),MethodFilter))
@@ -264,26 +295,17 @@ class TestMemoryLogger(TestCase):
         from xlutils.filter import MemoryLogger
         self.filter = MemoryLogger('somepath',['workbook'])
 
-    def test_method(self):
-        import xlutils.filter
-        try:
-            o = xlutils.filter.h
-            xlutils.filter.h = h = TestTraversable()
-            self.filter.method('name','foo',1)
-        finally:
-            xlutils.filter.h = o
-        self.assertEqual(len(h.children),1)
-        self.assertEqual(tuple(h.children)[0].path,
-                         ".heap().stat.dump('somepath')")
+    @replace('xlutils.filter.h',Mock())
+    def test_method(self,h):
+        self.filter.method('name','foo',1)
+        # h.heap().stat.dump('somepath')
+        compare(h.method_calls,[('heap',(),{})])
+        h = h.heap.return_value
+        compare(h.method_calls,[('stat.dump', ('somepath',),{})])
     
+    @replace('xlutils.filter.h',None)
     def test_method_no_heapy(self):
-        import xlutils.filter
-        try:
-            o = xlutils.filter.h
-            xlutils.filter.h = None
-            self.filter.method('name','foo',1)
-        finally:
-            xlutils.filter.h = o
+        self.filter.method('name','foo',1)
     
     def test_inheritance(self):
         self.failUnless(isinstance(self.filter,MethodFilter))
@@ -292,8 +314,8 @@ from xlutils.filter import ErrorFilter
 
 class TestErrorFilter(TestCase):
 
-    def test_set_rdsheet_1(self):
-        h = InstalledHandler('')
+    @log_capture()
+    def test_set_rdsheet_1(self,h):
         r = TestReader(
             ('Sheet1',[['S1R0C0']]),
             ('Sheet2',[[(XL_CELL_ERROR,0)]]),
@@ -301,26 +323,22 @@ class TestErrorFilter(TestCase):
         book = tuple(r.get_workbooks())[0][0]
         # fire methods on filter
         f = ErrorFilter()
-        f.next = c = TestCallable()
+        f.next = c = Mock()
+        f.start()
         f.workbook(book,'new.xls')
         f.sheet(book.sheet_by_index(0),'new')
         f.cell(0,0,0,0)
         f.set_rdsheet(book.sheet_by_index(1))
         f.cell(0,0,1,0)
         f.finish()
-        compare(self,c.called,[])
-        self.assertEqual(len(h.records),2)
-        self.assertEqual(
-            h.records[0].getMessage(),
-            "Cell A1 of sheet 'Sheet2' contains a bad value: error (#NULL!)"
-            )
-        self.assertEqual(
-            h.records[1].getMessage(),
-            'No output as errors have occurred.'
+        compare(c.method_calls,[])
+        h.check(
+            ('xlutils.filter','ERROR',"Cell A1 of sheet 'Sheet2' contains a bad value: error (#NULL!)"),
+            ('xlutils.filter','ERROR','No output as errors have occurred.'),
             )
 
-    def test_set_rdsheet_2(self):
-        h = InstalledHandler('')
+    @log_capture()
+    def test_set_rdsheet_2(self,h):
         r = TestReader(
             ('Sheet1',[['S1R0C0']]),
             ('Sheet2',[[(XL_CELL_ERROR,0)]]),
@@ -328,33 +346,35 @@ class TestErrorFilter(TestCase):
         book = tuple(r.get_workbooks())[0][0]
         # fire methods on filter
         f = ErrorFilter()
-        f.next = c = TestCallable()
+        f.next = c = Mock()
+        f.start()
         f.workbook(book,'new.xls')
         f.sheet(book.sheet_by_index(0),'new')
         f.cell(0,0,0,0)
         f.cell(0,0,1,0)
         f.finish()
-        compare(self,c.called,[
-            ('workbook', (O('xlrd.Book'), 'new.xls')),
-            ('sheet', (O('xlrd.sheet.Sheet'), u'new')),
-            ('row', (0, 0)),
-            ('cell', (0, 0, 0, 0)),
-            ('row', (1, 1)),
-            ('cell', (1, 0, 1, 0)),
-            ('finish', ())
+        compare(c.method_calls,[
+            ('start', (), {}),
+            ('workbook', (C('xlrd.Book'), 'new.xls'),{}),
+            ('sheet', (C('xlrd.sheet.Sheet',name='new',strict=False), u'new'),{}),
+            ('row', (0, 0),{}),
+            ('cell', (0, 0, 0, 0),{}),
+            ('row', (1, 1),{}),
+            ('cell', (1, 0, 1, 0),{}),
+            ('finish', (), {})
             ])
-        self.assertEqual(c.called[1][1][0].name,'new')
         self.assertEqual(len(h.records),0)
     
-    def test_multiple_workbooks_with_same_name(self):
-        h = InstalledHandler('')
+    @log_capture()
+    def test_multiple_workbooks_with_same_name(self,h):
         r = TestReader(
             ('Sheet1',[['S1R0C0']]),
             )
         book = tuple(r.get_workbooks())[0][0]
         # fire methods on filter
         f = ErrorFilter()
-        f.next = c = TestCallable()
+        f.next = c = Mock()
+        f.start()
         f.workbook(book,'new.xls')
         f.sheet(book.sheet_by_index(0),'new1')
         f.cell(0,0,0,0)
@@ -362,27 +382,72 @@ class TestErrorFilter(TestCase):
         f.sheet(book.sheet_by_index(0),'new2')
         f.cell(0,0,0,0)
         f.finish()
-        compare(self,c.called,[
-            ('workbook', (O('xlrd.Book'), 'new.xls')),
-            ('sheet', (O('xlrd.sheet.Sheet'), u'new1')),
-            ('row', (0, 0)),
-            ('cell', (0, 0, 0, 0)),
-            ('workbook', (O('xlrd.Book'), 'new.xls')),
-            ('sheet', (O('xlrd.sheet.Sheet'), u'new2')),
-            ('row', (0, 0)),
-            ('cell', (0, 0, 0, 0)),
-            ('finish', ())
+        compare(c.method_calls,[
+            ('start', (), {}),
+            ('workbook', (C('xlrd.Book'), 'new.xls'),{}),
+            ('sheet', (C('xlrd.sheet.Sheet',name='new1',strict=False), u'new1'),{}),
+            ('row', (0, 0),{}),
+            ('cell', (0, 0, 0, 0),{}),
+            ('workbook', (C('xlrd.Book'), 'new.xls'),{}),
+            ('sheet', (C('xlrd.sheet.Sheet',name='new2',strict=False), u'new2'),{}),
+            ('row', (0, 0),{}),
+            ('cell', (0, 0, 0, 0),{}),
+            ('finish', (), {})
             ])
-        self.assertEqual(c.called[1][1][0].name,'new1')
-        self.assertEqual(c.called[5][1][0].name,'new2')
         self.assertEqual(len(h.records),0)
     
+    def test_finish_resets(self):
+        # ...that's `start`s job!
+        r = TestReader(
+            ('Sheet1',[[(XL_CELL_ERROR,0)]]),
+            )
+        book = tuple(r.get_workbooks())[0][0]
+        # fire methods on filter
+        f = ErrorFilter()
+        f.next = c = Mock()
+        f.start()
+        f.workbook(book,'new.xls')
+        f.sheet(book.sheet_by_index(0),'new1')
+        f.cell(0,0,0,0)
+        self.assertTrue(f.handler.fired)
+        f.finish()
+        compare(c.method_calls,[])
+        self.assertFalse(f.handler.fired)
+        compare(f.temp_path,None)
+
+    @tempdir()
+    def test_start(self,d):
+        f = ErrorFilter()
+        f.next = m = Mock()
+        f.wtbook = 'junk'
+        f.handler.fired = 'junk'
+        f.temp_path = d.path
+        f.prefix = 'junk'
+        j = open(os.path.join(d.path,'junk.xls'),'wb')
+        j.write('junk')
+        j.close()
+
+        f.start()
+
+        compare(f.wtbook,None)
+        compare(f.handler.fired,False)
+        self.failIf(os.path.exists(d.path))
+        compare(os.listdir(f.temp_path),[])
+        compare(f.prefix,0)
+
+        f.finish()
+        
+        compare(m.method_calls,[
+            ('start', (), {}),
+            ('finish', (), {})
+            ])
+
 from xlutils.filter import ColumnTrimmer
 
 class TestColumnTrimmer(TestCase):
 
-    def test_set_rdsheet(self):
-        h = InstalledHandler('')
+    @log_capture()
+    def test_set_rdsheet_cols(self,h):
         r = TestReader(
             ('Sheet1',[['X',' ']]),
             ('Sheet2',[['X','X']]),
@@ -390,7 +455,8 @@ class TestColumnTrimmer(TestCase):
         book = tuple(r.get_workbooks())[0][0]
         # fire methods on filter
         f = ColumnTrimmer()
-        f.next = c = TestCallable()
+        f.next = c = Mock()
+        f.start()
         f.workbook(book,'new.xls')
         f.sheet(book.sheet_by_index(0),'new')
         f.row(0,0)
@@ -398,89 +464,172 @@ class TestColumnTrimmer(TestCase):
         f.set_rdsheet(book.sheet_by_index(1))
         f.cell(0,0,0,1)
         f.finish()
-        compare(self,c.called,[
-            ('workbook', (O('xlutils.tests.fixtures.DummyBook'), 'new.xls')),
-            ('sheet', (O('xlrd.sheet.Sheet'), u'new')),
-            ('row', (0, 0)),
-            ('cell', (0, 0, 0, 0)),
-            ('set_rdsheet', (O('xlrd.sheet.Sheet'),)),
-            ('cell', (0, 0, 0, 1)),
-            ('finish', ())
+        compare(c.method_calls,[
+            ('start', (), {}),
+            ('workbook', (C('xlutils.tests.fixtures.DummyBook'), 'new.xls'),{}),
+            ('sheet', (C('xlrd.sheet.Sheet',name='Sheet1',strict=False), u'new'),{}),
+            ('row', (0, 0),{}),
+            ('cell', (0, 0, 0, 0),{}),
+            ('set_rdsheet', (C('xlrd.sheet.Sheet',name='Sheet2',strict=False),),{}),
+            ('cell', (0, 0, 0, 1),{}),
+            ('finish', (), {})
             ])
-        self.assertEqual(c.called[1][1][0].name,'Sheet1')
-        self.assertEqual(c.called[4][1][0].name,'Sheet2')
         self.assertEqual(len(h.records),0)
 
-    def test_use_write_sheet_name_in_logging(self):
-        h = InstalledHandler('')
+    def test_set_rdsheet_rows(self):
+        r = TestReader(
+            ('Sheet1',[['X',' ']]),
+            ('Sheet2',[['X','X'],['X','X'],['X','X']]),
+            )
+        book = tuple(r.get_workbooks())[0][0]
+        # fire methods on filter
+        f = ColumnTrimmer()
+        f.next = c = Mock()
+        f.start()
+        f.workbook(book,'new.xls')
+        f.sheet(book.sheet_by_index(0),'new')
+        f.row(0,0)
+        f.cell(0,0,0,0)
+        f.set_rdsheet(book.sheet_by_index(1))
+        f.cell(2,0,1,0)
+        f.finish()
+        compare(c.method_calls,[
+            ('start', (), {}),
+            ('workbook', (C('xlutils.tests.fixtures.DummyBook'), 'new.xls'),{}),
+            ('sheet', (C('xlrd.sheet.Sheet',name='Sheet1',strict=False), u'new'),{}),
+            ('row', (0, 0),{}),
+            ('cell', (0, 0, 0, 0),{}),
+            ('set_rdsheet', (C('xlrd.sheet.Sheet',name='Sheet2',strict=False),),{}),
+            ('cell', (2, 0, 1, 0),{}),
+            ('finish', (), {})
+            ])
+
+    def test_set_rdsheet_trim(self):
+        r = TestReader(
+            ('Sheet1',[['X',' ']]),
+            ('Sheet2',[['X','X']]),
+            )
+        book = tuple(r.get_workbooks())[0][0]
+        # fire methods on filter
+        f = ColumnTrimmer()
+        f.next = c = Mock()
+        f.start()
+        f.workbook(book,'new.xls')
+        f.sheet(book.sheet_by_index(0),'new')
+        f.row(0,0)
+        f.cell(0,0,0,0)
+        f.cell(0,1,0,1)
+        f.set_rdsheet(book.sheet_by_index(1))
+        f.cell(0,0,1,0)
+        f.cell(0,1,1,1)
+        f.finish()
+        compare(c.method_calls,[
+            ('start', (), {}),
+            ('workbook', (C('xlutils.tests.fixtures.DummyBook'), 'new.xls'),{}),
+            ('sheet', (C('xlrd.sheet.Sheet',name='Sheet1',strict=False), u'new'),{}),
+            ('row', (0, 0),{}),
+            ('cell', (0, 0, 0, 0),{}),
+            ('cell', (0, 1, 0, 1),{}),
+            ('set_rdsheet', (C('xlrd.sheet.Sheet',name='Sheet2',strict=False),),{}),
+            ('cell', (0, 0, 1, 0),{}),
+            ('cell', (0, 1, 1, 1),{}),
+            ('finish', (), {})
+            ])
+
+    @log_capture()
+    def test_use_write_sheet_name_in_logging(self,h):
         r = TestReader(
             ('Sheet1',[['X',' ']]),
             )
         book = tuple(r.get_workbooks())[0][0]
         # fire methods on filter
         f = ColumnTrimmer()
-        f.next = c = TestCallable()
+        f.next = c = Mock()
+        f.start()
         f.workbook(book,'new.xls')
         f.sheet(book.sheet_by_index(0),'new')
         f.row(0,0)
         f.cell(0,0,0,0)
         f.cell(0,1,0,1)
         f.finish()
-        compare(self,c.called,[
-            ('workbook', (O('xlutils.tests.fixtures.DummyBook'), 'new.xls')),
-            ('sheet', (O('xlrd.sheet.Sheet'), u'new')),
-            ('row', (0, 0)),
-            ('cell', (0, 0, 0, 0)),
-            ('finish', ())
+        compare(c.method_calls,[
+            ('start', (), {}),
+            ('workbook', (C('xlutils.tests.fixtures.DummyBook'), 'new.xls'),{}),
+            ('sheet', (C('xlrd.sheet.Sheet',name='Sheet1',strict=False), u'new'),{}),
+            ('row', (0, 0),{}),
+            ('cell', (0, 0, 0, 0),{}),
+            ('finish', (),{})
             ])
-        self.assertEqual(c.called[1][1][0].name,'Sheet1')
-        self.assertEqual(len(h.records),1)
-        self.assertEqual(h.records[0].getMessage(),
-                         "Number of columns trimmed from 2 to 1 for sheet 'new'")
-        
+        h.check((
+            'xlutils.filter',
+            'DEBUG',
+            "Number of columns trimmed from 2 to 1 for sheet 'new'"
+                ))
 
-    def test_multiple_books(self):
-        h = InstalledHandler('')
+    @log_capture()
+    def test_multiple_books(self,h):
         r = GlobReader(os.path.join(test_files,'*.xls'))
         book = tuple(r.get_workbooks())[0][0]
         # fire methods on filter
         f = ColumnTrimmer()
-        f.next = c = TestCallable()
+        f.next = c = Mock()
         r(f)
-        compare(self,c.called,[
-            ('workbook', (O('xlrd.Book'), 'test.xls')),
-            ('sheet', (O('xlrd.sheet.Sheet'), u'Sheet1')),
-            ('row', (0, 0)),
-            ('row', (1, 1)),
-            ('cell', (0, 0, 0, 0)),('cell', (0, 1, 0, 1)),
-            ('cell', (1, 0, 1, 0)),('cell', (1, 1, 1, 1)),
-            ('sheet', (O('xlrd.sheet.Sheet'), u'Sheet2')),
-            ('row', (0, 0)),
-            ('row', (1, 1)),
-            ('cell', (0, 0, 0, 0)),('cell', (0, 1, 0, 1)),
-            ('cell', (1, 0, 1, 0)),('cell', (1, 1, 1, 1)),
-            ('workbook', (O('xlrd.Book'), 'testall.xls')),
-            ('sheet', (O('xlrd.sheet.Sheet'), u'Sheet1')),
-            ('row', (0, 0)),
-            ('row', (1, 1)),
-            ('row', (2, 2)),
-            ('row', (3, 3)),
-            ('row', (4, 4)),
-            ('row', (5, 5)),
-            ('cell', (0, 0, 0, 0)),('cell', (0, 1, 0, 1)),
-            ('cell', (1, 0, 1, 0)),('cell', (1, 1, 1, 1)),
-            ('cell', (2, 0, 2, 0)),('cell', (2, 1, 2, 1)),
-            ('cell', (3, 0, 3, 0)),('cell', (3, 1, 3, 1)),
-            ('cell', (4, 0, 4, 0)),('cell', (4, 1, 4, 1)),
-            ('cell', (5, 0, 5, 0)),('cell', (5, 1, 5, 1)),
-            ('sheet', (O('xlrd.sheet.Sheet'), u'Sheet2')),
-            ('row', (0, 0)),
-            ('row', (1, 1)),
-            ('cell', (0, 0, 0, 0)),('cell', (0, 1, 0, 1)),
-            ('cell', (1, 0, 1, 0)),('cell', (1, 1, 1, 1)),
-            ('finish', ())
+        compare(c.method_calls,[
+            ('start', (), {}),
+            ('workbook', (C('xlrd.Book'), 'test.xls'),{}),
+            ('sheet', (C('xlrd.sheet.Sheet'), u'Sheet1'),{}),
+            ('row', (0, 0),{}),
+            ('row', (1, 1),{}),
+            ('cell', (0, 0, 0, 0),{}),('cell', (0, 1, 0, 1),{}),
+            ('cell', (1, 0, 1, 0),{}),('cell', (1, 1, 1, 1),{}),
+            ('sheet', (C('xlrd.sheet.Sheet'), u'Sheet2'),{}),
+            ('row', (0, 0),{}),
+            ('row', (1, 1),{}),
+            ('cell', (0, 0, 0, 0),{}),('cell', (0, 1, 0, 1),{}),
+            ('cell', (1, 0, 1, 0),{}),('cell', (1, 1, 1, 1),{}),
+            ('workbook', (C('xlrd.Book'), 'testall.xls'),{}),
+            ('sheet', (C('xlrd.sheet.Sheet'), u'Sheet1'),{}),
+            ('row', (0, 0),{}),
+            ('row', (1, 1),{}),
+            ('row', (2, 2),{}),
+            ('row', (3, 3),{}),
+            ('row', (4, 4),{}),
+            ('row', (5, 5),{}),
+            ('cell', (0, 0, 0, 0),{}),('cell', (0, 1, 0, 1),{}),
+            ('cell', (1, 0, 1, 0),{}),('cell', (1, 1, 1, 1),{}),
+            ('cell', (2, 0, 2, 0),{}),('cell', (2, 1, 2, 1),{}),
+            ('cell', (3, 0, 3, 0),{}),('cell', (3, 1, 3, 1),{}),
+            ('cell', (4, 0, 4, 0),{}),('cell', (4, 1, 4, 1),{}),
+            ('cell', (5, 0, 5, 0),{}),('cell', (5, 1, 5, 1),{}),
+            ('sheet', (C('xlrd.sheet.Sheet'), u'Sheet2'),{}),
+            ('row', (0, 0),{}),
+            ('row', (1, 1),{}),
+            ('cell', (0, 0, 0, 0),{}),('cell', (0, 1, 0, 1),{}),
+            ('cell', (1, 0, 1, 0),{}),('cell', (1, 1, 1, 1),{}),
+            ('finish', (), {})
             ])
         self.assertEqual(len(h.records),0)
+
+    def test_start(self):
+        f = ColumnTrimmer()
+        f.next = m = Mock()
+        f.rdsheet = 'junk'
+        f.pending_rdsheet = 'junk'
+        f.rows = 'junk'
+        f.max_nonjunk = 'junk'
+        f.max = 'junk'
+
+        f.start()
+
+        compare(f.rdsheet,None)
+        compare(f.pending_rdsheet,None)
+        compare(f.rows,{})
+        compare(f.max_nonjunk,0)
+        compare(f.max,0)
+
+        compare(m.method_calls,[
+            ('start', (), {})
+            ])
     
 class CloseableTemporaryFile:
     def __init__(self,parent,filename):
@@ -553,7 +702,7 @@ class TestBaseWriter(TestCase):
             # /BUG
             
             # order doesn't matter in this list
-            compare(self,sorted(ash.merged_cells),sorted(es.merged_cells))
+            compare(sorted(ash.merged_cells),sorted(es.merged_cells))
 
             assertEqual(
                 ash,es,
@@ -749,6 +898,7 @@ class TestBaseWriter(TestCase):
         book = tuple(r.get_workbooks())[0][0]
         # fire methods on writer
         w = TestWriter()
+        w.start()
         w.workbook(book,'new.xls')
         w.sheet(book.sheet_by_index(0),'new')
         w.row(0,0)
@@ -775,6 +925,12 @@ class TestBaseWriter(TestCase):
         self.assertEqual(sheet.cell(2,0).value,'S1R1C0')
         self.assertEqual(sheet.cell(3,0).value,'S2R1C0')
         
+    def test_start(self):
+        w = TestWriter()
+        w.wtbook = 'junk'
+        w.start()
+        compare(w.wtbook,None)
+    
     def test_bogus_sheet_name(self):
         r = TestReader(
             ('sheet',([['S1R0C0']]),),
@@ -782,7 +938,9 @@ class TestBaseWriter(TestCase):
             )
         book = tuple(r.get_workbooks())[0][0]
         # fire methods on writer
-        self.assertRaises(ValueError,r,TestWriter())
+        should_raise(r,ValueError(
+            "A sheet named 'sheet' has already been added!"
+            ))(TestWriter())
     
     def test_empty_sheet_name(self):
         r = TestReader(
@@ -790,7 +948,9 @@ class TestBaseWriter(TestCase):
             )
         book = tuple(r.get_workbooks())[0][0]
         # fire methods on writer
-        self.assertRaises(ValueError,r,TestWriter())
+        should_raise(r,ValueError(
+            'Empty sheet name will result in invalid Excel file!'
+            ))(TestWriter())
     
     def test_max_length_sheet_name(self):
         name = 'X'*31
@@ -811,27 +971,30 @@ class TestBaseWriter(TestCase):
             )
         book = tuple(r.get_workbooks())[0][0]
         # fire methods on writer
-        self.assertRaises(ValueError,r,TestWriter())
+        should_raise(r,ValueError(
+            'Sheet name cannot be more than 31 characters long, '
+            'supplied name was 32 characters long!'
+            ))(TestWriter())
 
 class TestDirectoryWriter(TestCase):
 
-    def test_plus_in_workbook_name(self):
+    @tempdir
+    def test_plus_in_workbook_name(self,d):
         from xlutils.filter import DirectoryWriter
         r = TestReader(
             ('Sheet1',[['Cell']]),
             )
         book = tuple(r.get_workbooks())[0][0]
         # fire methods on writer
-        d = mkdtemp()
-        w = DirectoryWriter(d)
+        w = DirectoryWriter(d.path)
+        w.start()
         w.workbook(book,'a+file.xls')
         w.sheet(book.sheet_by_index(0),'new')
         w.row(0,0)
         w.cell(0,0,0,0)
         w.finish()
         # check file exists with the right name
-        self.assertEqual(os.listdir(d),['a+file.xls'])
-        rmtree(d)
+        self.assertEqual(os.listdir(d.path),['a+file.xls'])
 
 class TestProcess(TestCase):
 
@@ -839,13 +1002,13 @@ class TestProcess(TestCase):
         class DummyReader:
             def __call__(self,filter):
                 filter.finished()
-        F1 = TestCallable()
-        F2 = TestCallable()
+        F1 = Mock()
+        F2 = Mock()
         process(DummyReader(),F1,F2)
         self.failUnless(F1.next is F2)
-        self.failUnless(isinstance(F2.next,TestCallableMethod))
-        F1.compare(self,[('finished',())])
-        F2.compare(self,())
+        self.failUnless(isinstance(F2.next,Mock))
+        compare(F1.method_calls,[('finished',(),{})])
+        compare(F2.method_calls,[])
     
 class TestExamples(TestCase):
 
